from unittest import mock

from django.core.urlresolvers import reverse

from rest_framework import status
from rest_framework.test import APIRequestFactory
from rest_framework.test import APITestCase

from djabberd import views

from . import handlers as mock_handlers


factory = APIRequestFactory()


class AuthViewTests(APITestCase):
    def setUp(self):
        self.url = reverse("djabberd:auth")
        self.args = '?username={}&password={}'

    def test_username_missing(self):
        url = self.url + self.args.format('username', '')
        request = factory.get(url)

        response = views.AuthView().as_view()(request)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_password_missing(self):
        url = self.url + self.args.format('', 'password')
        request = factory.get(url)

        response = views.AuthView().as_view()(request)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    # To make testing easy setting use `username` to control the
    # result of `user_authentication`

    def test_http_200(self):
        url = self.url + self.args.format('True', 'password')
        request = factory.get(url)

        response = views.AuthView().as_view()(request)

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_http_401(self):
        url = self.url + self.args.format('False', 'password')
        request = factory.get(url)

        response = views.AuthView().as_view()(request)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)


class UserViewTests(APITestCase):
    def setUp(self):
        self.url = reverse("djabberd:user")
        self.args = '?username={}'

    def test_username_missing(self):
        url = self.url + self.args.format('')
        request = factory.get(url)

        response = views.UserView().as_view()(request)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    # To make testing easy setting use `username` to control the
    # result of `user_authentication`

    def test_user_exist(self):
        url = self.url + self.args.format('True')
        request = factory.get(url)

        response = views.UserView().as_view()(request)

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_user_does_not_exist(self):
        url = self.url + self.args.format('False')
        request = factory.get(url)

        response = views.UserView().as_view()(request)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)


class RosterViewTests(APITestCase):
    def setUp(self):
        self.url = reverse("djabberd:roster")
        self.args = '?username={}'

    def test_username_missing(self):
        url = self.url + self.args.format('')
        request = factory.get(url)

        response = views.RosterView().as_view()(request)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_user_exist(self):
        url = self.url + self.args.format('True')
        request = factory.get(url)

        response = views.RosterView().as_view()(request)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, mock_handlers.TEST_ROSTER)

    def test_user_does_not_exist(self):
        url = self.url + self.args.format('False')
        request = factory.get(url)

        response = views.RosterView().as_view()(request)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)


class ArchivePostViewTests(APITestCase):
    def setUp(self):
        self.url = reverse("djabberd:archive")

    def test_http_200(self):
        data = mock_handlers.TEST_ARCHIVE
        request = factory.post(self.url, data=data, format='json')

        with mock.patch('djabberd.handlers.archive_store') as mock_handler:
            response = views.ArchiveView().as_view()(request)
            mock_handler.assert_called_with(data)

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class ArchiveGetViewTests(APITestCase):
    def setUp(self):
        self.url = reverse("djabberd:archive")
        self.args = '?username={}'

    def test_username_missing(self):
        url = self.url + self.args.format('')
        request = factory.get(url)

        response = views.ArchiveView().as_view()(request)

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_with_username(self):
        username = 'True'
        url = self.url + self.args.format(username)
        request = factory.get(url)

        with mock.patch('djabberd.handlers.archive_get', autospec=True) as mh:
            mh.return_value = mock_handlers.TEST_ARCHIVE_GET
            response = views.ArchiveView().as_view()(request)
            mh.assert_called_with(username)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, mock_handlers.TEST_ARCHIVE_GET)

    def test_other_args_used(self):
        args = 'peer', 'after', 'before', 'limit', 'chat_type'
        username = 'True'
        for arg in args:
            url = self.url + self.args.format(username) + '&' + arg + '=asdf'
            self._test_arg(url, username, arg)

    def _test_arg(self, url, username, arg):
        with mock.patch('djabberd.handlers.archive_get', autospec=True) as mh:
            request = factory.get(url)
            views.ArchiveView().as_view()(request)
            mh.assert_called_with(username, **{arg: 'asdf'})



# Load handlers defined in `settings.DJABBERD_API_HANDLERS`

import importlib
import sys

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


HANDLERS_KEY = 'DJABBERD_API_HANDLERS'
HANDLERS_NAMES = (
    'user_authentication',
    'user_exists',
    'retrieve_user_roster',
    'archive_store',
    'archive_get',
)


def _load_handlers():
    handlers_module_name = getattr(settings, HANDLERS_KEY, None)
    if handlers_module_name is None:
        raise ImproperlyConfigured(
            "The {} setting can not be empty.".format(HANDLERS_KEY))

    handlers_module = importlib.import_module(handlers_module_name)

    module = sys.modules[__name__]
    for handler in HANDLERS_NAMES:
        new_handler = getattr(handlers_module, handler)
        setattr(module, handler, new_handler)


_load_handlers()



import imp
import importlib
from unittest import TestCase

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from . import handlers as mock_handlers


class LoadHandlersTests(TestCase):
    def test_no_handler_in_settings(self):
        """Raise `ImproperlyConfigured` if no handlers module is setup in settings

        """
        with self.assertRaises(ImproperlyConfigured) as cm:
            handlers = importlib.import_module('djabberd.handlers')
            del settings.DJABBERD_API_HANDLERS

            imp.reload(handlers)

            self.assertIn("can not be empty", cm.exception.msg)

    def test_valid_handlers(self):
        handlers = importlib.import_module('djabberd.handlers')

        for handler_name in handlers.HANDLERS_NAMES:
            self.assertIs(getattr(handlers, handler_name),
                          getattr(mock_handlers, handler_name))



from django.conf.urls import url

from . import views


app_name = 'djabberd'

urlpatterns = [
    url(r'^auth/$', views.AuthView.as_view(), name='auth'),
    url(r'^user/$', views.UserView.as_view(), name='user'),
    url(r'^roster/$', views.RosterView.as_view(), name='roster'),
    url(r'^archive/$', views.ArchiveView.as_view(), name='archive'),
]



DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:'},
}

SECRET_KEY = 'fake-key'

INSTALLED_APPS = ('django.contrib.auth',
                  'django.contrib.contenttypes',
                  'django.contrib.sessions',
                  'django.contrib.admin',
                  'django.contrib.sites',
                  'tests', )

ROOT_URLCONF = 'tests.urls'

DJABBERD_API_HANDLERS = 'tests.handlers'



import os
from setuptools import find_packages, setup

import djabberd

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    readme_lines = readme.readlines()
description = readme_lines[0].strip()
README = ''.join(readme_lines)


setup(
    name='djabberd',
    version=djabberd.__version__,
    include_package_data=not False,
    packages=find_packages(),
    license='GPL v3',
    description=description,
    long_description=README,
    author='Alexandre Varas',
    author_email='alej0varas@gmail.com',
    url='https://github.com/alej0varas/djabberd',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'Django>=1.8,<2',
        'djangorestframework>=3.2,<4',
    ],
)



# Test handlers


TEST_ARCHIVE = {"username": "USERNAME1", "peer": "USERNAME2",
                "xml": "MESSAGE_STANZA", "body": "TEXT",
                "timestamp": "TIMESTAMP", "direction": "DIRECTION"}
TEST_ARCHIVE_GET = {"archive": [{"username": "USERNAME1", "peer": "USERNAME2",
                                 "xml": "MESSAGE_STANZA", "body": "TEXT",
                                 "timestamp": "TIMESTAMP",
                                 "direction": "DIRECTION"}], "count": 1}
TEST_ROSTER = {"roster": [{"username": "CONTACT", "subscription": "both",
                           "nick": "NICK"}]}


def user_authentication(username, password):
    result = eval(username)
    return result


def user_exists(username):
    result = eval(username)
    return result


def retrieve_user_roster(username):
    user = eval(username)
    if user:
        return TEST_ROSTER


def archive_store(payload):
    pass


def archive_get(username, peer=None, after=None, before=None, limit=None, chat_type=None):
    return TEST_ARCHIVE_GET



from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from djabberd import handlers

# Authentication and permission have been explicitly disabled because
# we *don't* want the default ones. The only way to control access to
# the API is using IP address validation.


class InsecureAPIView(APIView):

    authentication_classes = ()
    permission_classes = ()


class AuthView(InsecureAPIView):
    """Validate `username` and `password` using `user_authentication`
    handler.

    """
    def get(self, request, format=None):
        username = request.GET.get('username')
        password = request.GET.get('password')
        if not (username and password):
            return Response(status=status.HTTP_401_UNAUTHORIZED)

        # We have a `username` and `password` and we pass them to the handler
        result = handlers.user_authentication(username, password)

        if result is True:
            return Response()

        return Response(status=status.HTTP_401_UNAUTHORIZED)


class UserView(InsecureAPIView):
    """Validate that the user with `username` exists using `user_exists`
    handler.

    """
    def get(self, request, format=None):
        username = request.GET.get('username')
        if not username:
            return Response(status=status.HTTP_401_UNAUTHORIZED)

        # We have a `username` and we pass them to the handler
        result = handlers.user_exists(username)

        if result is True:
            return Response()

        return Response(status=status.HTTP_401_UNAUTHORIZED)


class RosterView(InsecureAPIView):
    """Retrieves the user's roster for the user with the given `username`
    using `retrieve_user_roster` handler.

    """
    def get(self, request, format=None):
        username = request.GET.get('username')
        if not username:
            return Response(status=status.HTTP_401_UNAUTHORIZED)

        # We have a `username` and we pass it to the handler
        result = handlers.retrieve_user_roster(username)

        if result is not None:
            return Response(result)

        return Response(status=status.HTTP_401_UNAUTHORIZED)


class ArchiveView(InsecureAPIView):

    def get(self, request, format=None):
        """Retrieves the user's archive for the user with the given `username`
        using `archive_get` handler.

        """
        username = request.GET.get('username')
        if not username:
            return Response(status=status.HTTP_401_UNAUTHORIZED)

        # Get args from query string to pass the to the handler
        args = dict(list(request.GET.copy().items()))
        # We don't want `username` in args later
        args.pop('username')
        # Rename `type` to `chat_type`
        chat_type = args.pop('type', None)
        if chat_type is not None:
            args['chat_type'] = chat_type

        archive = handlers.archive_get(username, **args)

        return Response(archive)

    def post(self, request, format=None):
        """Stores the messages sent in the body

        """
        handlers.archive_store(request.data)

        return Response()
